@@ -1,6 +1,6 @@
 /* query-editor.h
  *
- * Copyright (C) 2004 Vivien Malerba
+ * Copyright (C) 2004 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,9 +21,9 @@
 #ifndef __QUERY_EDITOR__
 #define __QUERY_EDITOR__
 
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <libgnomedb/libgnomedb.h>
-#include <gnome.h>
 
 G_BEGIN_DECLS
 
@@ -49,14 +49,14 @@ struct _QueryEditor
 /* struct for the object's class */
 struct _QueryEditorClass
 {
-	GtkVBoxClass      parent_class;
+	GtkVBoxClass     parent_class;
 };
 
 /* 
  * Generic widget's methods 
 */
 GType      query_editor_get_type            (void);
-GtkWidget *query_editor_new                 (GnomeDbQuery *query);
+GtkWidget *query_editor_new                 (GdaQuery *query);
 
 G_END_DECLS
 
