@@ -1,6 +1,6 @@
 /* query-params-editor.c
  *
- * Copyright (C) 2004 - 2006 Vivien Malerba
+ * Copyright (C) 2004 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,6 +20,7 @@
 
 #include "query-params-editor.h"
 #include <glib/gi18n-lib.h>
+#include <libgda/gda-parameter-util.h>
 
 static void query_params_editor_class_init (QueryParamsEditorClass * class);
 static void query_params_editor_init (QueryParamsEditor * wid);
@@ -436,7 +437,7 @@ renderer_param_value_edited_cb (GtkCellRendererText *renderer,
 			GdaDataHandler *dh;
 			GdaDictType *dtype;
 
-			dtype = gda_query_field_value_get_dict_type (GDA_QUERY_FIELD_VALUE (field));
+			dtype = gda_entity_field_get_dict_type (GDA_ENTITY_FIELD (field));
 			if (dtype)
 				vtype = gda_dict_type_get_g_type (dtype);
 			else
@@ -444,7 +445,7 @@ renderer_param_value_edited_cb (GtkCellRendererText *renderer,
 
 			dh = gda_dict_get_default_handler (gda_object_get_dict (GDA_OBJECT (field)), vtype);
 			value = gda_data_handler_get_value_from_sql (dh, new_text, 
-								    gda_query_field_value_get_g_type (GDA_QUERY_FIELD_VALUE (field)));
+								     gda_entity_field_get_g_type (GDA_ENTITY_FIELD (field)));
 			if (value) {
 				gda_query_field_value_set_value (GDA_QUERY_FIELD_VALUE (field), value);
 				gda_value_free (value);
@@ -494,7 +495,7 @@ renderer_param_type_edited_cb (GtkCellRendererText *renderer,
 			field = (GdaQueryField *) (list->data);
 			
 			if (GDA_IS_QUERY_FIELD_VALUE (field)) 
-				gda_query_field_value_set_dict_type (GDA_QUERY_FIELD_VALUE (field), dtype);
+				gda_entity_field_set_dict_type (GDA_ENTITY_FIELD (field), dtype);
 			
 			list = g_slist_next (list);
 		}
@@ -656,7 +657,8 @@ query_changed_cb (GdaQuery *query, QueryParamsEditor *pedit)
 
 		list = fields;
 		while (list) {
-			if (GDA_IS_QUERY_FIELD_VALUE (list->data) && !gda_query_field_value_is_parameter (GDA_QUERY_FIELD_VALUE (list->data))) {
+			if (GDA_IS_QUERY_FIELD_VALUE (list->data) && 
+			    !gda_query_field_value_get_is_parameter (GDA_QUERY_FIELD_VALUE (list->data))) {
 				GdaQueryFieldValue *qfield = GDA_QUERY_FIELD_VALUE (list->data);
 				gchar *valstr;
 				const gchar *typestr;
@@ -685,7 +687,7 @@ query_changed_cb (GdaQuery *query, QueryParamsEditor *pedit)
 				else
 					valstr = g_strdup ("");
 				
-				typestr = gda_dict_type_get_sqlname (gda_query_field_value_get_dict_type (qfield));
+				typestr = gda_dict_type_get_sqlname (gda_entity_field_get_dict_type (GDA_ENTITY_FIELD (qfield)));
 				gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 						    COLUMN_PARAM_PTR, NULL,
 						    COLUMN_QFVALUE_PTR, qfield,
