@@ -1,6 +1,6 @@
 /* Mergeant
  *
- * Copyright (C) 1999 - 2006 Vivien Malerba
+ * Copyright (C) 1999 - 2007 Vivien Malerba
  * Copyright (C) 2002 - 2003 Rodrigo Moya
  *
  * Authors:
@@ -28,14 +28,15 @@
 #include <gtk/gtkstatusbar.h>
 #include <gtk/gtktable.h>
 #include <glib/gi18n-lib.h>
-#include <libgnomeui/gnome-about.h>
 #include <libgnomedb/libgnomedb.h>
 #include <libgnomedb/gnome-db-login-dialog.h>
-#include <libgnomedb/gnome-db-dbms-update-viewer.h>
+#include <libgnomedb-extra/libgnomedb-extra.h>
+#include <libgnomedb-extra/gnome-db-sql-console.h>
 #include "workspace.h"
 #include "workspace-window.h"
 #include <libgda/gda-threader.h>
 #include "utils.h"
+#include "binreloc/mg-binreloc.h"
 
 #define PROGRESS_BAR_TIMEOUT 20
 
@@ -48,8 +49,7 @@ typedef struct {
 	GdaDict        *dict;
 	
 	GSList         *consoles; /* list of GtkDialog for Sql consoles */
-
-	GdaTransaction *pending_transaction;
+	GtkWidget      *transaction_status;
 } WorkspaceWindowPrivate;
 
 static GList         *opened_workspaces = NULL;
@@ -67,17 +67,15 @@ destroy_private_data_cb (WorkspaceWindowPrivate *priv)
 		while (priv->consoles) 
 			gtk_widget_destroy (GTK_WIDGET (priv->consoles->data));
 
+		if (priv->transaction_status)
+			gtk_widget_destroy (priv->transaction_status);
+
 		if (priv->dict) {
 			gda_dict_save (priv->dict, NULL); /* save before destruction */
 			g_object_unref (priv->dict);
 			priv->dict = NULL;
 		}
 
-		if (priv->pending_transaction) {
-			g_object_unref (priv->pending_transaction);
-			priv->pending_transaction = NULL;
-		}
-
 		g_free (priv);
 
 		if (!opened_workspaces)
@@ -97,12 +95,101 @@ on_file_copy_workspace (GtkAction *action, WorkspaceWindowPrivate *priv)
 	create_new_workspace_window (priv->dict);
 }
 
+static gboolean
+treat_close_request (WorkspaceWindowPrivate *priv)
+{
+	GtkWidget *window = priv->window;
+	GdaConnection *cnc;
+	GdaTransactionStatus *tstatus;
+	gboolean do_close = TRUE;
+	GList *list;
+
+	/* if the connection is used by another workspace, then allow closing of this window */
+	for (list = opened_workspaces; list; list = list->next) {
+		if (((WorkspaceWindowPrivate*) list->data != priv) &&
+		    (((WorkspaceWindowPrivate*) list->data)->dict == priv->dict))
+			return TRUE;
+	}
+	
+	/* if a transaction is started, then try to commit it */
+	g_assert (priv);
+	cnc = gda_dict_get_connection (priv->dict);
+	g_assert (cnc);
+	g_assert (gda_connection_is_opened (cnc));
+	tstatus = gda_connection_get_transaction_status (cnc);
+	if (tstatus) {
+		gchar *msg;
+		GtkWidget *dlg;
+		gint result;
+
+		msg = g_strdup_printf (_("<b>A transaction has been started on the %s connection.</b>\n\n"
+					 "if it is not committed, any change since the beginning of the "
+					 "transaction will be lost. \n\n"
+					 "<u>Rollback</u> the transaction to cancel any change made since the "
+					 "transaction was started, <u>commit</u> the transaction to save those changes, "
+					 "or <u>ignore</u> the transaction to let the DBMS handle the transaction (this "
+					 "usually leads to lose any change). What do you want to do?"), 
+				       gda_connection_get_dsn (cnc));
+		dlg = gtk_message_dialog_new_with_markup (GTK_WINDOW (window), GTK_DIALOG_MODAL,
+							  GTK_MESSAGE_WARNING,
+							  GTK_BUTTONS_NONE, msg);
+		g_free (msg);
+		gtk_dialog_add_buttons (GTK_DIALOG (dlg), _("Rollback transaction"), 1,
+					_("Commit transaction"), 2, 
+					_("Ignore transaction"), 3, NULL);
+		result = gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
+		if ((result == 1) || (result == 2)) {
+			GError *error = NULL;
+
+			dlg = NULL;
+			if (result == 1) {
+				if (!gda_connection_rollback_transaction (cnc, NULL, &error))
+					dlg = gtk_message_dialog_new_with_markup (GTK_WINDOW (window), GTK_DIALOG_MODAL,
+										  GTK_MESSAGE_ERROR,
+										  GTK_BUTTONS_NONE, 
+										  _("<b>Could not rollback current transaction:</b>"
+										    "\n%s\n\nWhat do you want to do?"),
+										  error && error->message ? 
+										  error->message : _("No detail"));
+			}
+			else {
+				if (gda_connection_commit_transaction (cnc, NULL, &error))
+					dlg = gtk_message_dialog_new_with_markup (GTK_WINDOW (window), GTK_DIALOG_MODAL,
+										  GTK_MESSAGE_ERROR,
+										  GTK_BUTTONS_NONE, 
+										  _("<b>Could not commit current transaction:</b>"
+										    "\n%s\n\nWhat do you want to do?"),
+										  error && error->message ? 
+										  error->message : _("No detail"));
+			}
+
+			if (error)
+				g_error_free (error);
+			if (dlg) {
+				gtk_dialog_add_buttons (GTK_DIALOG (dlg), _("Close anyway"), 1,
+							_("Don't close this workspace window"), 2, NULL);
+				result = gtk_dialog_run (GTK_DIALOG (dlg));
+				do_close = (result == 1) ? TRUE : FALSE;
+				gtk_widget_destroy (dlg);
+			}
+		}
+	}
+
+	return do_close;
+}
+
+static gboolean
+delete_event( GtkWidget *widget, GdkEvent *event, WorkspaceWindowPrivate *priv )
+{
+	return !treat_close_request (priv);
+}
+
 static void
 on_file_close (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
-	GtkWidget *window = priv->window;
-	
-	gtk_widget_destroy (window);
+	if (treat_close_request (priv))
+		gtk_widget_destroy (priv->window);
 }
 
 static void
@@ -135,20 +222,49 @@ on_database_sync (GtkAction *action, WorkspaceWindowPrivate *priv)
 }
 
 static void
-on_database_begin (GtkAction *action, WorkspaceWindowPrivate *priv)
+transations_dlg_response_cb (GtkWidget *dlg, gint response_id, WorkspaceWindowPrivate *priv)
 {
+	gtk_widget_destroy (dlg);
+	priv->transaction_status = NULL;
 }
 
 static void
-on_database_commit (GtkAction *action, WorkspaceWindowPrivate *priv)
+on_transaction (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
-}
+	if (! priv->transaction_status) {
+		GtkWidget *dlg, *status;
+		gchar *title;
+		GdaConnection *cnc;
 
-static void
-on_database_rollback (GtkAction *action, WorkspaceWindowPrivate *priv)
-{
-}
+		/* dialog */
+		dlg = gtk_dialog_new_with_buttons (_("Transactions management"),
+						   NULL,
+						   0, GTK_STOCK_CLOSE,
+						   GTK_RESPONSE_ACCEPT, NULL);
+		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 
+		cnc = gda_dict_get_connection (priv->dict);
+		if (gda_connection_get_username (cnc))
+			title = g_strdup_printf ("Transactions management: %s@%s", gda_connection_get_username (cnc),
+						 gda_connection_get_dsn (cnc));
+		else
+			title = g_strdup_printf ("Transactions management: %s", gda_connection_get_dsn (cnc));
+		
+		gtk_window_set_title (GTK_WINDOW (dlg), title);
+		g_free (title);
+
+		/* transactions status widget */
+		status = gnome_db_transaction_status_new (gda_dict_get_connection (priv->dict));
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), status, TRUE, TRUE, 0);
+		gtk_widget_set_size_request (status, 400, 350);
+		g_signal_connect (G_OBJECT (dlg), "response",
+				  G_CALLBACK (transations_dlg_response_cb), priv);
+		gtk_widget_show_all (dlg);
+		priv->transaction_status = dlg;
+	}
+	else
+		gdk_window_raise (priv->transaction_status->window);
+}
 
 static void
 on_database_new (GtkAction *action, WorkspaceWindowPrivate *priv)
@@ -187,7 +303,7 @@ on_tools_data_sources (GtkAction *action, WorkspaceWindowPrivate *priv)
 	GError *error = NULL;
 
 	/* run gnome-database-properties dictig tool */
-	argv[0] = (char *) "gnome-database-properties";
+	argv[0] = (char *) "gnome-database-properties-3.0";
         argv[1] = NULL;
                                                                                   
 	if (!g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
@@ -222,10 +338,10 @@ on_sql_console (GtkAction *action, WorkspaceWindowPrivate *priv)
 	
 	cnc = gda_dict_get_connection (priv->dict);
 	if (gda_connection_get_username (cnc))
-                title = g_strdup_printf ("%s@%s - Mergeant", gda_connection_get_username (cnc),
+                title = g_strdup_printf ("SQL console: %s@%s", gda_connection_get_username (cnc),
                                          gda_connection_get_dsn (cnc));
         else
-                title = g_strdup_printf ("%s - Mergeant", gda_connection_get_dsn (cnc));
+                title = g_strdup_printf ("SQL console: %s", gda_connection_get_dsn (cnc));
 	
 	gtk_window_set_title (GTK_WINDOW (dlg), title);
 	g_free (title);
@@ -261,9 +377,9 @@ console_destroyed_cb (GtkWidget *widget, WorkspaceWindowPrivate *priv)
 }
 
 static void
-on_help_about (GtkAction *action, void *data)
+on_help_about (GtkAction *action, WorkspaceWindowPrivate *priv)
 {
-	GtkWidget *dialog = NULL;                                                                                    
+	GtkWidget *dialog = NULL;                                          
 	GdkPixbuf *icon;
 	const gchar *authors[] = {
 		"Vivien Malerba <malerba@gnome-db.org>",
@@ -311,13 +427,18 @@ on_help_about (GtkAction *action, void *data)
 		"Trinh Minh Thanh <tmthanh@linuxmail.org> Vietnamese translation\n" \
 		"Fan Zhang <a17841@bigpond.com> Simplified Chinese translations\n"
 		; 
-                                                                                    
-	icon = gdk_pixbuf_new_from_file (MERGEANT_PIXMAPDIR "/mergeant.png", NULL);
+        
+	gchar *file;
+	
+	/*file = gnome_db_gbr_get_icon_path ("gnome-db.png");*/
+	file = mg_gbr_get_file_path (MG_DATA_DIR, "pixmaps", "mergeant.png", NULL);
+        icon = gdk_pixbuf_new_from_file (file, NULL);
+        g_free (file);
 		
 	dialog = gtk_about_dialog_new ();
 	gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (dialog), PACKAGE);
 	gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (dialog), PACKAGE_VERSION);
-	gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (dialog), "(C) 1998-2005 GNOME Foundation");
+	gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (dialog), "(C) 1998-2007 GNOME Foundation");
 	gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (dialog), _("Database services for the GNOME Desktop"));
 	gtk_about_dialog_set_license (GTK_ABOUT_DIALOG (dialog), "GNU General Public License");
 	gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (dialog), "http://www.gnome-db.org");
@@ -325,10 +446,9 @@ on_help_about (GtkAction *action, void *data)
 	gtk_about_dialog_set_documenters (GTK_ABOUT_DIALOG (dialog), documenters);
 	gtk_about_dialog_set_translator_credits (GTK_ABOUT_DIALOG (dialog), translator_credits);
 	gtk_about_dialog_set_logo (GTK_ABOUT_DIALOG (dialog), icon);
-	g_signal_connect (G_OBJECT (dialog), "close",
-			  G_CALLBACK (gtk_widget_destroyed), &dialog);
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (data));
-	gtk_widget_show (dialog);
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (priv->window));
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
 }
 
 static GtkActionEntry ui_actions[] = {
@@ -340,9 +460,7 @@ static GtkActionEntry ui_actions[] = {
 	{ "FileExit", GTK_STOCK_QUIT, "_Quit", NULL, "Exit the application", G_CALLBACK (on_file_exit) },
 	{ "Database", NULL, "_Database", NULL, "Database", NULL },
 	{ "DatabaseSync", GTK_STOCK_REFRESH, "_Sync", NULL, "Refresh metadata from database", G_CALLBACK (on_database_sync) },
-	{ "DatabaseBeginTransaction", NULL, "_Begin", NULL, "Start a new transaction", G_CALLBACK (on_database_begin) },
-	{ "DatabaseCommitTransaction", NULL, "_Commit", NULL, "Commit current transaction", G_CALLBACK (on_database_commit) },
-	{ "DatabaseRollbackTransaction", NULL, "_Rollback", NULL, "Discard current transaction", G_CALLBACK (on_database_rollback) },
+	{ "Transactions", NULL, "_Transations", NULL, "Transactions management", G_CALLBACK (on_transaction) },
 	{ "DatabaseCreate", GTK_STOCK_NEW, "_Create a database", NULL, "Create a new database", G_CALLBACK (on_database_new) },
 	{ "DatabaseDelete", GTK_STOCK_DELETE, "_Delete a database", NULL, "Delete an existing database", G_CALLBACK (on_database_del) },
 	{ "ToolsDataSources", GTK_STOCK_PREFERENCES, "_Datasources", NULL, "Manage the list of declared datasources", G_CALLBACK (on_tools_data_sources) },
@@ -369,9 +487,7 @@ static const gchar *ui_actions_info =
         "      <separator/>"
         "      <menuitem name='DatabaseSync' action= 'DatabaseSync'/>"
         "      <separator/>"
-        "      <menuitem name='DatabaseBeginTransaction' action= 'DatabaseBeginTransaction'/>"
-        "      <menuitem name='DatabaseCommitTransaction' action= 'DatabaseCommitTransaction'/>"
-        "      <menuitem name='DatabaseRollbackTransaction' action= 'DatabaseRollbackTransaction'/>"
+        "      <menuitem name='Transactions' action= 'Transactions'/>"
         "      <separator/>"
         "      <menuitem name='SqlConsole' action='SqlConsole'/>"
         "    </menu>"
@@ -510,10 +626,15 @@ create_new_workspace_window_with_dict (GdaDict *dict, gboolean force_dbms_update
 
 	/* create the window */
 	priv->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	g_signal_connect (G_OBJECT (priv->window), "delete_event",
+			  G_CALLBACK (delete_event), priv);
 	gtk_window_set_title (GTK_WINDOW (priv->window), "Mergeant");
 	g_object_set_data_full (G_OBJECT (priv->window), "Mergeant_WorkspaceWindowPrivate",
 				priv, (GDestroyNotify) destroy_private_data_cb);
-	gtk_window_set_icon_from_file (GTK_WINDOW (priv->window), MERGEANT_PIXMAPDIR "/mergeant.png", NULL);
+	gchar *str;
+	str = mg_gbr_get_file_path (MG_DATA_DIR, "pixmaps", "mergeant.png", NULL);
+	gtk_window_set_icon_from_file (GTK_WINDOW (priv->window), str, NULL);
+	g_free (str);
 
 	cnc = gda_dict_get_connection (priv->dict);
         if (gda_connection_get_username (cnc))
@@ -765,6 +886,20 @@ thread_load_dict_finished (GdaThreader *thread, guint job_id, ThreadData *thdata
 			gchar *msg;
 			
 			gtk_widget_hide (dsn_dialog);
+			/* limit the size of error message to 10 lines */
+			if (error->message) {
+				gchar *ptr;
+				gint nlnb = 10;
+				for (ptr = error->message; ptr && (ptr-error->message < 500) && (nlnb >= 0); ptr++) {
+					if (*ptr == '\n')
+						nlnb--;
+				}
+				if (*ptr) {
+					/* truncate error message */
+					ptr -= 7;
+					strcpy (ptr, "\n[...]");
+				}
+			}
 			msg = g_markup_printf_escaped (_("<b><big>Failed loading data dictionary</big></b>\n"
 							 "<small>The previously saved data dictionary for this connection "
 							 "can't be loaded because it contains errors:\n%s</small>\n\n"
