@@ -1,6 +1,6 @@
 /* ws-queries.c
  *
- * Copyright (C) 2004 - 2006 Vivien Malerba
+ * Copyright (C) 2004 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,12 +18,13 @@
  * USA
  */
 
+#include <glib/gi18n-lib.h>
+#include <libgnomedb-extra/libgnomedb-extra.h>
 #include "ws-queries.h"
 #include "workspace-page.h"
 #include <string.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
-#include <libgnomedb/libgnomedb.h>
 #include "query-druid.h"
 #include "query-editor.h"
 #include "query-params-editor.h"
@@ -453,14 +454,12 @@ static void
 action_query_add_cb (GtkAction *action, WsQueries *ws)
 {
 	GtkWidget *druid;
-	GtkWidget *window;
 
 	druid = query_druid_new (ws->priv->dict);
 	gtk_widget_show (druid);
-	gnome_druid_construct_with_window (GNOME_DRUID (druid), _("New Query"), NULL, TRUE, &window);
 
-	ws->priv->opened_dialogs = g_slist_append (ws->priv->opened_dialogs, window);
-	g_signal_connect (G_OBJECT (window), "destroy",
+	ws->priv->opened_dialogs = g_slist_append (ws->priv->opened_dialogs, druid);
+	g_signal_connect (G_OBJECT (druid), "destroy",
 			  G_CALLBACK (opened_dialog_closed_cb), ws);
 }
 
@@ -613,7 +612,7 @@ utility_query_execute_modif (GdaQuery *query, GdaParameterList *context,
 	while (parent_window && !GTK_IS_WINDOW (parent_window)) 
 		parent_window = gtk_widget_get_parent (parent_window);
 
-	sql = gda_renderer_render_as_sql (GDA_RENDERER (query), context, 0, &error);
+	sql = gda_renderer_render_as_sql (GDA_RENDERER (query), context, NULL, 0, &error);
 	qtype = gda_query_get_query_type (query);
 
 	switch (qtype) {
@@ -740,7 +739,7 @@ action_query_exec_cb (GtkAction *action, WsQueries *ws)
 	qei->query = gda_query_new_copy (query, NULL);
 	query = qei->query;
 
-	if (gda_query_is_modif_query (query)) {
+	if (gda_query_is_modify_query (query)) {
 		/* modification query */
 		context = gda_query_get_parameter_list (query);
 		if (fill_context_in_dialog (ws, context) == GTK_RESPONSE_ACCEPT) 
@@ -843,7 +842,7 @@ action_query_exec_cb (GtkAction *action, WsQueries *ws)
 				gchar *msg;
 				const gchar *sql;
 				
-				sql = gda_renderer_render_as_sql (GDA_RENDERER (query), context, 
+				sql = gda_renderer_render_as_sql (GDA_RENDERER (query), context, NULL,
 								  GDA_RENDERER_EXTRA_PRETTY_SQL, NULL);
 				query_exec_instance_free (qei);
 				msg = g_strdup_printf (_("<b><big>Execute the following query ?</big></b>\n"
@@ -957,7 +956,7 @@ query_info_display_update (GdaQuery *query, WsQueries *ws)
 	if (query) {
 		gchar *sql;
 		GError *error = NULL;
-		sql = gda_renderer_render_as_sql (GDA_RENDERER (query), NULL,
+		sql = gda_renderer_render_as_sql (GDA_RENDERER (query), NULL, NULL,
 						  GDA_RENDERER_EXTRA_PRETTY_SQL, &error);
 		if (sql) {
 			gnome_db_editor_set_text (GNOME_DB_EDITOR (ws->priv->sql_editor), sql, -1);
